package com.trafficparrot.examples.aws;

import com.github.tomakehurst.wiremock.client.VerificationException;
import com.github.tomakehurst.wiremock.client.WireMock;
import com.github.tomakehurst.wiremock.http.Request;
import com.github.tomakehurst.wiremock.matching.*;
import org.apache.http.NameValuePair;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.utils.URLEncodedUtils;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.message.BasicNameValuePair;
import org.junit.rules.TestWatcher;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import static com.github.tomakehurst.wiremock.matching.RequestPatternBuilder.forCustomMatcher;
import static java.nio.charset.StandardCharsets.UTF_8;
import static java.util.stream.Collectors.toMap;
import static org.assertj.core.api.Assertions.assertThat;
import static org.junit.Assert.fail;

public class TrafficParrotAmazonSQS extends TestWatcher {

    private final WireMock wireMock;
    private final int trafficParrotHttpPort;

    private TrafficParrotAmazonSQS(WireMock wireMock, int trafficParrotHttpPort) {
        this.wireMock = wireMock;
        this.trafficParrotHttpPort = trafficParrotHttpPort;
    }

    public static TrafficParrotAmazonSQS trafficParrotAmazonSQS(int trafficParrotHttpPort) {
        WireMock wireMock = WireMock.create().port(trafficParrotHttpPort).build();
        return new TrafficParrotAmazonSQS(wireMock, trafficParrotHttpPort);
    }

    public void verifyMessageWasSentToQueueContainingText(String queueName, String expected) {
        verifyMessageWasSentToQueue(queueName, new ContainsPattern(expected));
    }

    public void verifyMessageWasSentToQueueMatchingJsonPath(String queueName, String expected) {
        verifyMessageWasSentToQueue(queueName, new MatchesJsonPathPattern(expected));
    }

    public void purgeQueue(String queueName) throws IOException {
        try (CloseableHttpClient httpclient = HttpClients.createDefault()) {
            HttpPost request = new HttpPost("http://localhost:" + trafficParrotHttpPort + "/aws/sqs/queue/" + queueName);
            List<NameValuePair> params = new ArrayList<>();
            params.add(new BasicNameValuePair("Action", "PurgeQueue"));
            request.setEntity(new UrlEncodedFormEntity(params));

            try (CloseableHttpResponse response = httpclient.execute(request)) {
                assertThat(response.getStatusLine().getStatusCode()).isEqualTo(200);
            }
        }
    }

    private void verifyMessageWasSentToQueue(String queueName, StringValuePattern pattern) {
        try {
            wireMock.verifyThat(forCustomMatcher(new AmazonSQSMatcher(queueName, pattern)));
        } catch (VerificationException e) {
            fail("Could not find Amazon SQS request that " + pattern);
        }
    }

    private static class AmazonSQSMatcher implements ValueMatcher<Request> {
        private final String queueName;
        private final StringValuePattern pattern;

        public AmazonSQSMatcher(String queueName, StringValuePattern pattern) {
            this.queueName = queueName;
            this.pattern = pattern;
        }

        @Override
        public MatchResult match(Request request) {
            if (!request.getUrl().contains("/aws/sqs")) {
                return MatchResult.noMatch();
            }

            if (!request.getUrl().endsWith("queue/" + queueName)) {
                return MatchResult.noMatch();
            }

            Map<String, String> formParameters = URLEncodedUtils.parse(request.getBodyAsString(), UTF_8).stream().collect(toMap(NameValuePair::getName, NameValuePair::getValue));

            String action = formParameters.get("Action");
            if (!"SendMessage".equals(action)) {
                return MatchResult.noMatch();
            }

            String messageBody = formParameters.get("MessageBody");
            return pattern.match(messageBody);
        }
    }
}
